CodeQL / Analyze (actions) (push) Canceled after 0s
CodeQL / Analyze (python) (push) Canceled after 0s
pre-commit / pre-commit (push) Canceled after 0s
Multi-Arch CI / setup (push) Canceled after 0s
Publish build_manylinux_x86_64 images / publish_build_manylinux_x86_64 (push) Canceled after 0s
Setup (Multi-Arch) / setup (push) Canceled after 0s
Multi-Arch CI / Linux::${{ fromJSON(needs.setup.outputs.linux_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubi10 image / publish_no_rocm_image_ubi10 (push) Canceled after 0s
Multi-Arch CI / Windows::${{ fromJSON(needs.setup.outputs.windows_build_config || '{}').build_variant_label || 'skip' }} (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Multi-Arch CI / Manifest Diff (push) Canceled after 0s
TheRock Manifest Diff Report / Generate Manifest Diff Report (push) Canceled after 0s
Multi-Arch CI / CI Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_ocl_rt images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04_rocgdb images / publish_no_rocm_image_ubuntu24_04_rocgdb (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_ocl_rt (push) Canceled after 0s
Publish TheRock Docker image / build-and-push-image (push) Canceled after 0s
Unit Tests / Unit Tests :: ubuntu-24.04 (push) Canceled after 0s
Unit Tests / Unit Tests :: windows-2022 (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_media (push) Canceled after 0s
Unit Tests / Unit Tests Summary (push) Canceled after 0s
Publish no_rocm_image_ubuntu24_04 images / publish_no_rocm_image_ubuntu24_04_openmpi (push) Canceled after 0s
119 lines
3.7 KiB
C++
119 lines
3.7 KiB
C++
//===----------------------------------------------------------------------===//
|
|
//
|
|
// Part of the libcu++ Project, under the Apache License v2.0 with LLVM Exceptions.
|
|
// See https://llvm.org/LICENSE.txt for license information.
|
|
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
|
|
// SPDX-FileCopyrightText: Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES.
|
|
//
|
|
//===----------------------------------------------------------------------===//
|
|
|
|
// Modifications Copyright (c) 2024-2025 Advanced Micro Devices, Inc.
|
|
// Permission is hereby granted, free of charge, to any person obtaining a copy
|
|
// of this software and associated documentation files (the "Software"), to deal
|
|
// in the Software without restriction, including without limitation the rights
|
|
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
|
|
// copies of the Software, and to permit persons to whom the Software is
|
|
// furnished to do so, subject to the following conditions:
|
|
// The above copyright notice and this permission notice shall be included in
|
|
// all copies or substantial portions of the Software.
|
|
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
|
|
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
|
|
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
|
|
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
|
|
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
|
|
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
|
|
// THE SOFTWARE.
|
|
|
|
// UNSUPPORTED: nvrtc, hiprtc, pre-sm-70, hipcc
|
|
// UNSUPPORTED: true
|
|
|
|
// uncomment for a really verbose output detailing what test steps are being launched
|
|
// #define DEBUG_TESTERS
|
|
|
|
#include <cuda/barrier>
|
|
#include <cuda/std/cassert>
|
|
|
|
#include <atomic>
|
|
|
|
#include "helpers.h"
|
|
|
|
template <typename Barrier>
|
|
struct barrier_and_token
|
|
{
|
|
using barrier_t = Barrier;
|
|
using token_t = typename barrier_t::arrival_token;
|
|
|
|
barrier_t barrier;
|
|
cuda::std::atomic<bool> parity_waiting{false};
|
|
|
|
template <typename... Args>
|
|
__host__ __device__ barrier_and_token(Args&&... args)
|
|
: barrier{cuda::std::forward<Args>(args)...}
|
|
{}
|
|
};
|
|
|
|
struct barrier_arrive_and_wait
|
|
{
|
|
using async = cuda::std::true_type;
|
|
|
|
template <typename Data>
|
|
__host__ __device__ static void perform(Data& data)
|
|
{
|
|
while (data.parity_waiting.load(cuda::std::memory_order_acquire) == false)
|
|
{
|
|
data.parity_waiting.wait(false);
|
|
}
|
|
data.barrier.arrive_and_wait();
|
|
}
|
|
};
|
|
|
|
template <bool Phase>
|
|
struct barrier_parity_wait
|
|
{
|
|
using async = cuda::std::true_type;
|
|
|
|
template <typename Data>
|
|
__host__ __device__ static void perform(Data& data)
|
|
{
|
|
data.parity_waiting.store(true, cuda::std::memory_order_release);
|
|
data.parity_waiting.notify_all();
|
|
data.barrier.wait_parity(Phase);
|
|
}
|
|
};
|
|
|
|
struct clear_token
|
|
{
|
|
template <typename Data>
|
|
__host__ __device__ static void perform(Data& data)
|
|
{
|
|
data.parity_waiting.store(false, cuda::std::memory_order_release);
|
|
}
|
|
};
|
|
|
|
using aw_aw_pw1 =
|
|
performer_list<barrier_parity_wait<false>,
|
|
barrier_arrive_and_wait,
|
|
barrier_arrive_and_wait,
|
|
async_tester_fence,
|
|
clear_token>;
|
|
|
|
using aw_aw_pw2 =
|
|
performer_list<barrier_parity_wait<true>,
|
|
barrier_arrive_and_wait,
|
|
barrier_arrive_and_wait,
|
|
async_tester_fence,
|
|
clear_token>;
|
|
|
|
void kernel_invoker()
|
|
{
|
|
validate_pinned<barrier_and_token<cuda::std::barrier<>>, aw_aw_pw1>(2);
|
|
validate_pinned<barrier_and_token<cuda::std::barrier<>>, aw_aw_pw2>(2);
|
|
}
|
|
|
|
int main(int arg, char** argv)
|
|
{
|
|
NV_IF_TARGET(NV_IS_HOST, (kernel_invoker();))
|
|
|
|
return 0;
|
|
}
|